@@ -1,4 +1,4 @@
-/* $Id: gdict-pref-dialog.c,v 1.63 2005/01/18 19:33:52 vnoel Exp $ */
+/* $Id: gdict-pref-dialog.c,v 1.71 2005/10/06 14:06:45 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -18,6 +18,7 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
+#include <string.h>
 
 #include "gdict-pref-dialog.h"
 #include "gdict-app.h"
@@ -127,7 +128,7 @@ gdict_pref_dialog_class_init (GDictPrefDialogClass *class)
   object_class->finalize = gdict_pref_dialog_finalize;
 }
 
-static web_site_edited (GtkCellRendererText *cell,
+static void web_site_edited (GtkCellRendererText *cell,
                         gchar *path_str, gchar *new_text,
                         gpointer data)
 {
@@ -154,7 +155,7 @@ static web_site_edited (GtkCellRendererText *cell,
 	
 }
 
-static web_site_toggled (GtkCellRendererToggle *cell,
+static void web_site_toggled (GtkCellRendererToggle *cell,
                          gchar                 *path_str,
                          gpointer               data)
 {
@@ -177,7 +178,7 @@ static web_site_toggled (GtkCellRendererToggle *cell,
 	gtk_list_store_set (GTK_LIST_STORE (model), &iter, ENABLED, toggle_item, -1);
 }
 
-static web_site_add (GtkButton *widget, gpointer data)
+static void web_site_add (GtkButton *widget, gpointer data)
 {
 	GDictWebSearchSite *site;
 	GDictPrefDialog *pref_dialog = data;
@@ -205,7 +206,7 @@ static web_site_add (GtkButton *widget, gpointer data)
 	gtk_tree_path_free (path);
 }
 
-static web_site_remove (GtkButton *widget, gpointer data)
+static void web_site_remove (GtkButton *widget, gpointer data)
 {
 	GDictPrefDialog *pref_dialog = data;
 	GDictWindow *gdict = pref_dialog->gdict;
@@ -222,7 +223,7 @@ static web_site_remove (GtkButton *widget, gpointer data)
 	gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
 }
 
-static web_site_selected (GtkTreeSelection *selection, gpointer data)
+static void web_site_selected (GtkTreeSelection *selection, gpointer data)
 {
 	GDictPrefDialog *pref_dialog = data;
 	if (gtk_tree_selection_count_selected_rows (selection) > 0) 
@@ -231,16 +232,29 @@ static web_site_selected (GtkTreeSelection *selection, gpointer data)
 		gtk_widget_set_sensitive (pref_dialog->remove_button, FALSE);
 }
 
+void
+static set_font (GtkFontButton *widget, gpointer data)
+{
+	GDictPrefDialog *pref_dialog = data;
+	GDictWindow *gdict = pref_dialog->gdict;
+	const gchar *fontname;
+
+	fontname = gtk_font_button_get_font_name (widget);
+	gdict_defbox_set_font (gdict->defbox, fontname);
+}
+
 static GtkWidget *
 create_dictionary_panel (GDictPrefDialog *pref_dialog)
 {
   GtkWidget *vbox, *hbox;
+	GtkWidget *font_label, *font_button;
+	GtkWidget *menu_item;
 
   vbox = gtk_vbox_new (FALSE, 0);
   gtk_box_set_spacing (GTK_BOX (vbox), 2);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
     
-  pref_dialog->table = GTK_TABLE (gtk_table_new (4, 2, FALSE));
+  pref_dialog->table = GTK_TABLE (gtk_table_new (5, 2, FALSE));
   gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (pref_dialog->table), FALSE, FALSE, 0);
   gtk_table_set_row_spacings (pref_dialog->table, 6);
   gtk_table_set_col_spacings (pref_dialog->table, 12);
@@ -311,15 +325,32 @@ create_dictionary_panel (GDictPrefDialog *pref_dialog)
   gtk_misc_set_alignment (GTK_MISC (pref_dialog->db_label), 0, 0.5);
   gtk_table_attach_defaults (pref_dialog->table, pref_dialog->db_label, 0, 1, 2, 3);
   pref_dialog->db_list = GTK_MENU (gtk_menu_new ());
-  gtk_widget_show (GTK_WIDGET (pref_dialog->db_list));
+  pref_dialog->db_sel = GTK_OPTION_MENU (gtk_option_menu_new ());
+  gtk_table_attach_defaults (pref_dialog->table, GTK_WIDGET(pref_dialog->db_sel), 1, 2, 2, 3);
+  gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->db_sel), FALSE);
+	gtk_widget_show (GTK_WIDGET (pref_dialog->db_list));
     
   pref_dialog->strat_label = gtk_label_new_with_mnemonic (_("Strat_egy:"));
   gtk_label_set_justify (GTK_LABEL (pref_dialog->strat_label), GTK_JUSTIFY_LEFT);
   gtk_misc_set_alignment (GTK_MISC (pref_dialog->strat_label), 0, 0.5);
   gtk_table_attach_defaults (pref_dialog->table, pref_dialog->strat_label, 0, 1, 3, 4);
   pref_dialog->strat_list = GTK_MENU (gtk_menu_new ());
+  pref_dialog->strat_sel = GTK_OPTION_MENU (gtk_option_menu_new ());
+  gtk_table_attach_defaults (pref_dialog->table, GTK_WIDGET(pref_dialog->strat_sel), 1, 2, 3, 4);
+  gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->strat_sel), FALSE);
   gtk_widget_show (GTK_WIDGET (pref_dialog->strat_list));
-    
+
+	font_label = gtk_label_new_with_mnemonic (_("F_ont:"));
+	gtk_label_set_justify (GTK_LABEL (font_label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment (GTK_MISC (font_label), 0, 0.5);
+	gtk_table_attach_defaults (pref_dialog->table, font_label, 0, 1, 4, 5);
+	pref_dialog->font_button = gtk_font_button_new ();
+	gtk_table_attach_defaults (pref_dialog->table, pref_dialog->font_button, 1, 2, 4, 5);
+	g_signal_connect (G_OBJECT (pref_dialog->font_button), "font-set",
+										G_CALLBACK (set_font), pref_dialog);
+
+	gtk_label_set_mnemonic_widget (GTK_LABEL (font_label),
+	                               GTK_WIDGET (pref_dialog->font_button));	
   gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
 
   return vbox;
@@ -449,7 +480,6 @@ gdict_pref_dialog_set (GDictPrefDialog *pref_dialog)
                         POINTER_COLUMN, site,
                         -1);
 	}
-
 }
 
 static void 
@@ -485,10 +515,6 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
     
   g_signal_connect (G_OBJECT (pref_dialog), "response",
                     G_CALLBACK (response_cb), NULL);
-
-  /* To prevent dialog desctruction when pressing ESC */
-  g_signal_connect (G_OBJECT (pref_dialog), "delete_event", 
-                    G_CALLBACK (gtk_true), NULL);
 }
 
 static void
@@ -517,12 +543,15 @@ gdict_pref_dialog_new (GDictWindow *gdict)
   pref_dialog = GDICT_PREF_DIALOG (g_object_new (GDICT_TYPE_PREF_DIALOG, NULL));
   pref_dialog->gdict = gdict;
   pref_dialog->context = context;
+	if (gdict->defbox->fontname)
+		gtk_font_button_set_font_name (GTK_FONT_BUTTON(pref_dialog->font_button),
+																	 gdict->defbox->fontname);
   gdict_pref_dialog_set (pref_dialog);
 
   pref_dialog_reset_db (pref_dialog);
   pref_dialog_reset_strat (pref_dialog);
     
-  gtk_widget_show_all (GTK_WIDGET (pref_dialog));
+	gtk_widget_show_all (GTK_WIDGET (pref_dialog));
 
   return GTK_WIDGET (pref_dialog);
 }
@@ -576,10 +605,13 @@ static void
 pref_dialog_add_db (GDictPrefDialog *pref_dialog, gchar *db,
                     gchar *desc) 
 {
-  GtkWidget *menu_item;
+  GtkWidget *menu_item, *label;
   GDictWindow *gdict = pref_dialog->gdict;
     
   menu_item = gtk_menu_item_new_with_label (desc);
+  label = gtk_bin_get_child (GTK_BIN (menu_item));
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+  gtk_label_set_width_chars (GTK_LABEL (label), 30);
   g_signal_connect (G_OBJECT (menu_item), "activate", 
                     G_CALLBACK (pref_set_db_cb), pref_dialog);
   g_object_set_data (G_OBJECT (menu_item), "db_name", db);
@@ -601,10 +633,13 @@ static void
 pref_dialog_add_strat (GDictPrefDialog *pref_dialog, gchar *strat,
                        gchar *desc) 
 {
-  GtkWidget *menu_item;
+  GtkWidget *menu_item, *label;
   GDictWindow *gdict = pref_dialog->gdict;
     
   menu_item = gtk_menu_item_new_with_label (desc);
+  label = gtk_bin_get_child (GTK_BIN (menu_item));
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+  gtk_label_set_width_chars (GTK_LABEL (label), 30);
   g_signal_connect (G_OBJECT (menu_item), "activate", 
                     G_CALLBACK (pref_set_strat_cb), pref_dialog);
   g_object_set_data (G_OBJECT (menu_item), "strat_name", strat);
@@ -630,12 +665,8 @@ pref_dialog_reset_db (GDictPrefDialog *pref_dialog)
   if (pref_dialog->get_db_cmd)
     dict_command_destroy (pref_dialog->get_db_cmd);
     
-  if (pref_dialog->db_sel) {
+  if (pref_dialog->db_sel)
     gtk_option_menu_remove_menu (pref_dialog->db_sel);
-    gtk_widget_destroy (GTK_WIDGET (pref_dialog->db_sel));
-    gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
-    pref_dialog->db_sel = NULL;
-  }
 
   pref_dialog->db_list = GTK_MENU (gtk_menu_new ());
     
@@ -661,6 +692,8 @@ pref_dialog_reset_db (GDictPrefDialog *pref_dialog)
       /* Could not look up search strategies, so just display a
        * label; FIXME: Memory leak
        */
+      gtk_widget_destroy (GTK_WIDGET (pref_dialog->db_sel));
+      pref_dialog->db_sel = NULL;
       error_label1 = GTK_WIDGET (gtk_label_new (_("Cannot connect to server")));
       alignment1 = GTK_WIDGET (gtk_alignment_new (0, 0.5, 0, 0));
       gtk_container_add (GTK_CONTAINER (alignment1), error_label1);
@@ -681,12 +714,8 @@ pref_dialog_reset_strat (GDictPrefDialog *pref_dialog)
   if (pref_dialog->get_strat_cmd)
     dict_command_destroy (pref_dialog->get_strat_cmd);
     
-  if (pref_dialog->strat_sel) {
+  if (pref_dialog->strat_sel)
     gtk_option_menu_remove_menu (pref_dialog->strat_sel);
-    gtk_widget_destroy (GTK_WIDGET (pref_dialog->strat_sel));
-    gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
-    pref_dialog->strat_sel = NULL;
-  }
     
   pref_dialog->strat_list = GTK_MENU (gtk_menu_new ());
     
@@ -710,6 +739,8 @@ pref_dialog_reset_strat (GDictPrefDialog *pref_dialog)
       /* Could not look up search strategies, so just display a
        * label; FIXME: Memory leak
        */
+      gtk_widget_destroy (GTK_WIDGET (pref_dialog->strat_sel));
+      pref_dialog->strat_sel = NULL;
       error_label2 = GTK_WIDGET (gtk_label_new (_("Cannot connect to server")));
       alignment2 = GTK_WIDGET (gtk_alignment_new (0, 0.5, 0, 0));
       gtk_container_add (GTK_CONTAINER (alignment2), error_label2);
@@ -780,11 +811,11 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
                 int num_found, gpointer data)
 {
   GDictPrefDialog *pref_dialog;
-  GtkWidget *alignment;
+  GtkWidget *alignment, *label;
   GtkOptionMenu *option_menu = NULL;
   GtkMenu *use_menu = NULL;
-  gint row = 0;
   gboolean writable;
+  int row;
     
   g_return_if_fail (data != NULL);
   g_return_if_fail (GDICT_IS_PREF_DIALOG (data));
@@ -793,21 +824,25 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
     
   if (code == DICT_STATUS_OK) {
     if (command->cmd == C_SHOW_DB) {
-      row = 2;
       use_menu = pref_dialog->db_list;
+      option_menu = pref_dialog->db_sel;
+      row = 2;
     }
     else if (command->cmd == C_SHOW_STRAT) {
-      row = 3;
       use_menu = pref_dialog->strat_list;
+      option_menu = pref_dialog->strat_sel;
+      row = 3;
+    }
+
+    if (option_menu == NULL) {
+        option_menu = GTK_OPTION_MENU (gtk_option_menu_new ());      
+      	gtk_table_attach_defaults (pref_dialog->table, GTK_WIDGET(option_menu),
+                                   1, 2, row, row+1);
     }
         
-    option_menu = GTK_OPTION_MENU (gtk_option_menu_new ());
     gtk_option_menu_set_menu (option_menu, GTK_WIDGET (use_menu));
+    gtk_widget_show (GTK_WIDGET (option_menu));
 
-    gtk_table_attach_defaults (pref_dialog->table, GTK_WIDGET (option_menu), 
-                               1, 2, row, row + 1);
-    gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
-        
     if (command->cmd == C_SHOW_DB) {
 	    pref_dialog->db_sel = option_menu;
       gtk_label_set_mnemonic_widget (GTK_LABEL (pref_dialog->db_label),
